import pandas as pd
from colorama import Fore, Back, Style

import os
import sys
sys.path.insert(0, os.path.abspath('..'))

# from pprint import pprint
# pprint(sys.path)

from edabox import DataBox

# Test on Kaggle Titanic Dataset
df = pd.read_csv('./data/titanic.csv')

# dbx = box.DataBox(df, target=[])
dbx = DataBox(df, target=['Survived'])

dbx.look_inside()




import pandas as pd
from colorama import Fore, Back, Style

import os
import sys
sys.path.insert(0, os.path.abspath('..'))

# from pprint import pprint
# pprint(sys.path)

from edabox.core.utils import process_frame

# --------------------------------------------------------------------- #
# Test on Kaggle Titanic Dataset
df = pd.read_csv('./data/titanic.csv')
samples, cols = df.shape
target = ['Survived']
id = 'PassengerId'

# process_frame.get_shape(df, target=['Embarked', 'Embarked', 'globglob'])

process_frame.get_shape(df, target=['Embarked'])

process_frame.explore_target(df, target=['Survived'])

process_frame.explore_features(df, target=target, id=id)

# --------------------------------------------------------------------- #








import pandas as pd

import typing

from edabox.core.utils import process_frame


class DataBox:
    """
    DataBox class
    
    Parameters
    ----------
    df : pandas.DataFrame
    target : list or None, default None
        For datasets, used for predictive-modelling or classification, takes the name of the 'target' columns.
        It is highly recommended to specify the 'target' columns, in order to gain better insight into the dataset. 
    id : str or int or none, default None
      Unique identifier column for each row in a dataset.
      It is highly recommended to specify the 'id' columns, in order to gain better insight into the dataset.
    """
    def __init__(self, df : pd.DataFrame,
                 target : "list | None" = None,
                 id : "str | int | None" = None):
        self.df = df
        self.target = target
        self.id = id
    
    @property
    def frame(self) -> pd.DataFrame:
        return self.df

    def get_shape(self):
        process_frame.get_shape(self.df,self.target)

    def look_inside(self):
        """
        Takes a look inside a DataFrame
        """
        df = self.df

        self.get_shape()
        process_frame.explore_target(self.df, self.target)
        process_frame.explore_features(self.df, self.target)





import pandas as pd

from edabox.core.utils import process_frame
from edabox.core.utils import process_feature
from edabox.core import DataBox

# module level doc-string
__doc__ = """
edabox - a library for Python to gain insights into datasets
WORK IN PROGRESS
"""

__all__ = [
    "DataBox",
    "read_csv",
    "read_excel",
    "read_frame",
    "process_frame",
    "process_feature"
]


import pandas as pd

import numpy as np
from scipy import mean
from scipy import stats

import matplotlib.pyplot as plt
import seaborn as sns

from colorama import Fore, Back, Style

import copy
import re
from typing import List

from warnings import warn

# -------------------------------------------------------------------- #

foregd = Fore.CYAN
backgd = Back.BLACK
style  = Style.BRIGHT
style_reset = Style.RESET_ALL

# -------------------------------------------------------------------- #
"""
Functions
----------
    feature_vs_target()
    explore_feature()
"""
# -------------------------------------------------------------------- #

#function
def feature_vs_target() -> None:
    pass 

# function:
def explore_feature( df : pd.DataFrame,
                     feature : "str | int",
                     target : "List[str] | List[int] | None" = None,
                     visuals : "bool | None" = True ) -> None:

    """
    Explores the details of a feature in the DataFrame

    Parameters
    ----------
    df : pandas.DataFrame
    feature : str or list
    target : list<str> or list<int> or None, default None
    visuals : bool or None, default True
    """
    # when feature column-name is passed
    if type(feature) == str:
        df_feat = df.loc[:,[feature]]
        feat_name = feature
    # when feature column-number is passed
    elif type(feature) == int:
        df_feat = df.iloc[:,[feature]]
        feat_name = df_feat.columns[0]

    str1a  = "-------------- Feature  : "
    str1b  = " ----------------"
    str2   = "        Feature  : "
    str3   = "--               dtype  :  "
    str4   = "-- missing value count  :  "    
    str5   = "--  unique value count  :  "

    str6   = "--                 max  :  "
    str7   = "--                 min  :  "
    str8   = "--                mean  :  "
    str9   = "--         mode, count  :  "


    # print(f"{foregd}{style}{str1}{style_reset}")
    # print(f"{foregd}{style}{str2}{feat_name}{style_reset}")
    print(f"{foregd}{style}{str1a}{feat_name}{str1b}{style_reset}")   
    
    # data-type
    feat_dtype = df_feat[feat_name].dtypes
    print(f"{foregd}{style}{str3}{feat_dtype}{style_reset}")
    
    # missing values
    dfnull = df_feat.isnull()
    n_null = sum( dfnull[feat_name] == True )
    print(f"{foregd}{style}{str4}{n_null}{style_reset}")    

    # unique values
    vals = df_feat[feat_name].unique()
    n_unique = len(vals)
    print(f"{foregd}{style}{str5}{n_unique}{style_reset}")

    # if numeric column
    if 'int' in str(feat_dtype):
        feat_max  = int(df_feat[feat_name].max())
        feat_min  = int(df_feat[feat_name].min())
        feat_mean = int(np.mean(df_feat[feat_name]))
        feat_mode, feat_mode_cnt = stats.mode(df_feat[feat_name],     
                                               keepdims=False)
    elif 'float' in str(feat_dtype):
        feat_max  = df_feat[feat_name].max()
        feat_min  = df_feat[feat_name].min()
        feat_mean = np.mean(df_feat[feat_name])
        feat_mode = stats.mode(df_feat[feat_name],
                               keepdims=False)
    if re.search('int', str(feat_dtype)) or re.search('int', str(feat_dtype)):
        print(f"{foregd}{style}{str6}{feat_max}{style_reset}")
        print(f"{foregd}{style}{str7}{feat_min}{style_reset}")
        print(f"{foregd}{style}{str8}{feat_mean}{style_reset}")
        print(f"{foregd}{style}{str9}{feat_mode}, {feat_mode_cnt}{style_reset}")
    
    







from setuptools import find_packages, setup

setup(
    name='edabox',
    version='1.0.2.2',
    description='Python Library to Gain Insight into Datasets',
    author='Poushali Mukherjee',
    license='MIT',
    url='https://github.com/poushalimukherjee/edabox',
    packages=find_packages(),
    install_requires=['pandas','colorama', 'matplotlib', 'seaborn'],
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    test_suite='tests'
)


import pandas as pd

from colorama import Fore, Back, Style

import copy
import re
from typing import List

from warnings import warn

from edabox.core.utils import process_feature

# -------------------------------------------------------------------- #

foregd = Fore.CYAN
backgd = Back.BLACK
style  = Style.BRIGHT
style_reset = Style.RESET_ALL

# -------------------------------------------------------------------- #
"""
Functions
---------
    get_shape()
    explore_target()
    explore_features()
"""
# -------------------------------------------------------------------- #

# function: 
def get_shape(df : pd.DataFrame,
              target : "list | None" = None,
              id : "str | int | None" = None ) -> None:
    samples, cols = df.shape
    """
    Displayes the Sample, Feature and Target counts inside the DataFrame

    Parameters
    ----------
    df : pandas.DataFrame
    target : list or None, default None
    id : str or int or None, default None
    """

    str1 = "======================================"
    str2 = "Samples  : "
    str3 = "Columns  : "
    str4 = "Features : "
    str5 = "Targets  : "


    print(f"{foregd}{style}{str1}{style_reset}")
    print(f"{foregd}{style}{str2}{samples}{style_reset}")    

    if target == None:
        print(f"{foregd}{style}{str3}{cols}{style_reset}")
    else:
        target_valid = {el for el in target if el in df.columns}
        ntarget = len(target_valid)

        if ntarget < len(target):
            warn("'Target' names are either not in the dataset or entered more than once")

        features = cols - ntarget 
        str_targets = str(ntarget)

        print(f"{foregd}{style}{str4}{features}{style_reset}")
        print(f"{foregd}{style}{str5}{str_targets}{style_reset}")

    # print(f"{foregd}{style}{str1}{style_reset}")


# function: 
def explore_target(df : pd.DataFrame,
                   target : "list | None" = None ) -> None:
    
    """
    Displays the Target column details

    Parameters
    ----------
    df : pandas.DataFrame
    target : list or None, default None
    """
    if target != None:
       target_valid = {str(el) for el in target if el in df.columns}
       ntarget = len(target_valid)

       if ntarget == 0:
        return

       if ntarget < len(target):
        warn("'Target' names are either not in the dataset or entered more than once")

       str1 = "======================================"
       str2 = "            Target Classes            "
       str3 = "Binary Classes      : "
       str4 = "Multi-label Classes : "
       str5 = "Targets  : "

       print(f"{foregd}{style}{str1}{style_reset}")
       print(f"{foregd}{style}{str2}{style_reset}")        

       dict_target_bin  = {}
       dict_target_mult = {}
       for el in target_valid:
        label_cnt = df[el].nunique()
        df_label = df[[el]].drop_duplicates()
        if label_cnt == 2:
            dict_target_bin[el] = list(df_label[el])
        elif label_cnt > 2:
            dict_target_mult[el] = list(df_label[el])

        # print(f"{foregd}{style}{str3}{style_reset}")
        for key, val in dict_target_bin.items():
            str_key_val = "" + key + ": { "
            str_labels  = " -> "
            for el in val:
                label_cnt = sum( df[key] == el )
                str_labels = str_labels + str(label_cnt) + ", "
                str_key_val = str_key_val + str(el) + ", "

            str_labels = str_labels + "end"
            str_labels = re.sub(", end", "", str_labels)

            str_key_val = str_key_val + "}"
            str_key_val = re.sub(', }', ' }', str_key_val)

            str_key_val = str_key_val + str_labels
            print(f"{foregd}{style}{str_key_val}{style_reset}")


# function: 
def explore_features(df : pd.DataFrame,
                     target : "List[str] | List[int] | None" = None,
                     id : "str | int | None" = None,
                     features : "List[str] | List[int] | None" = None,
                     verbose : "bool | None" = False ) -> None:
    """
    Explores the feature-details of the DataFrame

    Parameters
    ----------
    df : pandas.DataFrame
    target : list or None, default None
    id : str or int or None, default None
    features : list<str> or list<int> or None, default None
    """
    df_feat = copy.deepcopy(df)
    if target != None:
        # when target column-names are passed
        if type(target) == List[str]:
          df_feat = df.drop(target, axis=1)
        # when target column-names are passed
        elif type(target) == List[str]:
            pass # WIP

    if id != None:
        # when ID column-name is passed
        if type(id) == str:
            df_feat = df_feat.drop([id], axis=1)
        # when ID column-number is passed
        elif type(id) == int:
            pass # WIP            
    
    if features != None:
        # when feature column-names are passed
        if type(features) == List(str):
            features_valid = [el for el in features if el in df.columns]
            df_feat = df[features_valid]
            if len(features_valid) < len(features):
                warn("One or more 'features' are either not in the dataset or entered more than once")
        # when feature column-numbers are passed
        elif type(features) == List(int):
            pass # WIP

    for feature in df_feat.columns:
        process_feature.explore_feature(df, feature=feature)